@@ -4,11 +4,11 @@
  * LICENSE-APACHE for a copy of the license.
  */
 #include <postgres.h>
-#include <fmgr.h>
-#include <miscadmin.h>
 #include <catalog/pg_collation.h>
 #include <executor/executor.h>
 #include <executor/nodeSubplan.h>
+#include <fmgr.h>
+#include <miscadmin.h>
 #include <nodes/bitmapset.h>
 #include <nodes/makefuncs.h>
 #include <nodes/nodeFuncs.h>
@@ -26,10 +26,11 @@
 
 #include <math.h>
 
-#include "nodes/chunk_append/chunk_append.h"
 #include "loader/lwlocks.h"
+#include "nodes/chunk_append/chunk_append.h"
 #include "planner/planner.h"
 #include "transform.h"
+#include "ts_catalog/chunk_column_stats.h"
 
 #define INVALID_SUBPLAN_INDEX (-1)
 #define NO_MATCHING_SUBPLANS (-2)
@@ -958,8 +959,8 @@ ts_constify_restrictinfos(PlannerInfo *root, List *restrictinfos)
 				additional_clause = ts_transform_cross_datatype_comparison(additional_clause);
 				additional_clause =
 					(Expr *) estimate_expression_value(root, (Node *) additional_clause);
-				additional_list = lappend(additional_list,
-										  make_simple_restrictinfo_compat(root, additional_clause));
+				additional_list =
+					lappend(additional_list, make_simple_restrictinfo(root, additional_clause));
 			}
 		}
 		rinfo->clause = constified;
@@ -1114,6 +1115,12 @@ ca_get_relation_constraints(Oid relationObjectId, Index varno, bool include_notn
 				}
 			}
 		}
+
+		/* Add column range min/max ranges in 'CHECK CONSTRAINT' form */
+		result = list_concat(result,
+							 ts_chunk_column_stats_construct_check_constraints(relation,
+																			   relationObjectId,
+																			   varno));
 	}
 
 	table_close(relation, NoLock);
